@@ -16,15 +16,18 @@ from pyglet.libs.win32.types import (
     MINMAXINFO,
     RAWINPUT,
     RAWINPUTHEADER,
+    RECT,
+    SIZE,
     TRACKMOUSEEVENT,
     WCHAR,
 )
 from pyglet.libs.win32.winkey import chmap, keymap
 
 if compat_platform not in ('cygwin', 'win32'):
-    raise ImportError('Not a win32 platform.')  # noqa: EM101
+    raise ImportError('Not a win32 platform.')
 
 from ctypes import (
+    POINTER,
     byref,
     c_int,
     c_int16,
@@ -39,7 +42,7 @@ from ctypes import (
 )
 
 import pyglet
-from pyglet.canvas.win32 import Win32Canvas
+from pyglet.display.win32 import Win32Canvas
 from pyglet.event import EventDispatcher
 from pyglet.libs.win32 import (
     BITMAPV5HEADER,
@@ -123,6 +126,8 @@ class Win32Window(BaseWindow):
     _pending_click: bool = False
     _in_title_bar: bool = False
 
+    _mouse_scale: float = 1
+
     _keyboard_state: dict[int, bool] = {0x02A: False, 0x036: False}  # For shift keys.
 
     _ws_style: int = 0
@@ -180,12 +185,22 @@ class Win32Window(BaseWindow):
         else:
             self._ws_style &= ~(constants.WS_THICKFRAME | constants.WS_MAXIMIZEBOX)
 
+        self._dpi = self._screen.get_dpi()
+
         if self._fullscreen:
             width = self.screen.width
             height = self.screen.height
         else:
+            if pyglet.options.dpi_scaling in ("scaled", "stretch"):
+                w, h = self.get_requested_size()
+                self._width = int(w * self.scale)
+                self._height = int(h * self.scale)
+
+                if pyglet.options.dpi_scaling == "stretch":
+                    self._mouse_scale = self.scale
+
             width, height = \
-                self._client_to_window_size(self._width, self._height)
+                self._client_to_window_size(self._width, self._height, self._dpi)
 
         if not self._window_class:
             module = _kernel32.GetModuleHandleW(None)
@@ -310,7 +325,7 @@ class Win32Window(BaseWindow):
         if self._visible:
             self.set_visible()
             # Might need resize event if going from fullscreen to fullscreen
-            self.dispatch_event('on_resize', self._width, self._height)
+            self.dispatch_event('_on_internal_resize', self._width, self._height)
             self.dispatch_event('on_expose')
 
     def _update_view_location(self, width: int, height: int) -> None:
@@ -398,23 +413,20 @@ class Win32Window(BaseWindow):
                               constants.SWP_NOOWNERZORDER))
 
     def get_location(self) -> tuple[int, int]:
-        rect = RECT()
-        _user32.GetClientRect(self._hwnd, byref(rect))
-        point = POINT()
-        point.x = rect.left
-        point.y = rect.top
+        point = POINT(0, 0)
         _user32.ClientToScreen(self._hwnd, byref(point))
         return point.x, point.y
 
     def set_size(self, width: int, height: int) -> None:
+        if pyglet.options.dpi_scaling in ("scaled", "stretch"):
+            width = int(width * self.scale)
+            height = int(height * self.scale)
+
         super().set_size(width, height)
-        width, height = self._client_to_window_size(width, height)
+        width, height = self._client_to_window_size_dpi(width, height)
         _user32.SetWindowPos(self._hwnd, 0, 0, 0, width, height,
                              (constants.SWP_NOZORDER | constants.SWP_NOMOVE | constants.SWP_NOOWNERZORDER))
-        self.dispatch_event('on_resize', self._width, self._height)
-
-    def get_size(self) -> tuple[int, int]:
-        return self._width, self._height
+        self.dispatch_event('_on_internal_resize', self._width, self._height)
 
     def set_minimum_size(self, width: int, height: int) -> None:
         self._minimum_size = width, height
@@ -430,7 +442,7 @@ class Win32Window(BaseWindow):
             insertAfter = constants.HWND_TOP
             _user32.SetWindowPos(self._hwnd, insertAfter, 0, 0, 0, 0,
                                  constants.SWP_NOMOVE | constants.SWP_NOSIZE | constants.SWP_SHOWWINDOW)
-            self.dispatch_event('on_resize', self._width, self._height)
+            self.dispatch_event('_on_internal_resize', self._width, self._height)
             self.activate()
             self.dispatch_event('on_show')
         else:
@@ -445,6 +457,13 @@ class Win32Window(BaseWindow):
     def maximize(self) -> None:
         _user32.ShowWindow(self._hwnd, constants.SW_MAXIMIZE)
 
+    def get_window_screen(self):
+        """ Gets the current screen the window is on.
+            If between monitors will retrieve the screen with the most screen space.
+        """
+        handle = _user32.MonitorFromWindow(self._hwnd, constants.MONITOR_DEFAULTTONEAREST)
+        return [screen for screen in self.display.get_screens() if screen._handle == handle][0]
+
     def set_caption(self, caption: str) -> None:
         self._caption = caption
         _user32.SetWindowTextW(self._hwnd, c_wchar_p(caption))
@@ -475,6 +494,15 @@ class Win32Window(BaseWindow):
 
         self._mouse_platform_visible = platform_visible
 
+    def get_size(self) -> tuple[int, int]:
+        if pyglet.options.dpi_scaling == "stretch":
+            return int(self._width / self.scale), int(self._height / self.scale)
+
+        return self._width, self._height
+
+    def get_framebuffer_size(self) -> tuple[int, int]:
+        return self._width, self._height
+
     def _set_cursor_visibility(self, platform_visible: bool) -> None:
         # Avoid calling ShowCursor with the current visibility (which would
         # push the counter too far away from zero).
@@ -589,7 +617,7 @@ class Win32Window(BaseWindow):
         # of any size, so no scaling necessary.
 
         def best_image(width: int, height: int) -> pyglet.image.ImageData:
-            # A heuristic for finding closest sized image to required size.
+            # A heuristic for finding the closest sized image to required size.
             b_image = images[0]
             for img in images:
                 if img.width == width and img.height == height:
@@ -599,7 +627,7 @@ class Win32Window(BaseWindow):
                         img.width * img.height > b_image.width * b_image.height:
                     # At least wide enough, and largest area
                     b_image = img
-            return image
+            return b_image
 
         def get_icon(img: pyglet.image.ImageData) -> HICON:
             # Alpha-blended icon: see http://support.microsoft.com/kb/318876
@@ -730,23 +758,50 @@ class Win32Window(BaseWindow):
         return text
 
     # Private util
-
-    def _client_to_window_size(self, width: int, height: int) -> tuple[int, int]:
+    def _client_to_window_size(self, width: int, height: int, dpi: int) -> tuple[int, int]:
         rect = RECT()
         rect.left = 0
         rect.top = 0
         rect.right = width
         rect.bottom = height
-        _user32.AdjustWindowRectEx(byref(rect),
-                                   self._ws_style, False, self._ex_ws_style)
+
+        if constants.WINDOWS_10_ANNIVERSARY_UPDATE_OR_GREATER:
+            _user32.AdjustWindowRectExForDpi(byref(rect),
+                                             self._ws_style, False, self._ex_ws_style, dpi)
+        else:
+            _user32.AdjustWindowRectEx(byref(rect),
+                                       self._ws_style, False, self._ex_ws_style)
+
+        return rect.right - rect.left, rect.bottom - rect.top
+
+    def _client_to_window_size_dpi(self, width, height):
+        """ This returns the true window size factoring in styles, borders, title bars.
+            Retrieves DPI directly from the Window hwnd, used after window creation.
+        """
+        rect = RECT()
+        rect.left = 0
+        rect.top = 0
+        rect.right = width
+        rect.bottom = height
+
+        if constants.WINDOWS_10_ANNIVERSARY_UPDATE_OR_GREATER:
+            _user32.AdjustWindowRectExForDpi(byref(rect),
+                self._ws_style, False, self._ex_ws_style, _user32.GetDpiForWindow(self._hwnd))
+        else:
+            _user32.AdjustWindowRectEx(byref(rect),
+                self._ws_style, False, self._ex_ws_style)
+
         return rect.right - rect.left, rect.bottom - rect.top
 
     def _client_to_window_pos(self, x: int, y: int) -> tuple[int, int]:
-        rect = RECT()
-        rect.left = x
-        rect.top = y
-        _user32.AdjustWindowRectEx(byref(rect),
-                                   self._ws_style, False, self._ex_ws_style)
+        rect = RECT(x, y, x, y)
+
+        if constants.WINDOWS_10_ANNIVERSARY_UPDATE_OR_GREATER:
+            _user32.AdjustWindowRectExForDpi(byref(rect),
+                self._ws_style, False, self._ex_ws_style, _user32.GetDpiForWindow(self._hwnd))
+        else:
+            _user32.AdjustWindowRectEx(byref(rect),
+                self._ws_style, False, self._ex_ws_style)
         return rect.left, rect.top
 
     # Event dispatching
@@ -909,12 +964,13 @@ class Win32Window(BaseWindow):
 
             if rmouse.usFlags & 0x01 == constants.MOUSE_MOVE_RELATIVE:
                 if rmouse.lLastX != 0 or rmouse.lLastY != 0:
+                    scale = self.scale
                     # Motion event
                     # In relative motion, Y axis is positive for below.
                     # We invert it for Pyglet so positive is motion up.
                     if self._exclusive_mouse_buttons:
                         self.dispatch_event('on_mouse_drag', 0, 0,
-                                            rmouse.lLastX, -rmouse.lLastY,
+                                            rmouse.lLastX * scale, -rmouse.lLastY * scale,
                                             self._exclusive_mouse_buttons,
                                             self._get_modifiers())
                     else:
@@ -983,7 +1039,7 @@ class Win32Window(BaseWindow):
             # re-entering (to track the next WM_MOUSELEAVE).
             self._mouse_in_window = True
             self.set_mouse_platform_visible()
-            self.dispatch_event('on_mouse_enter', x, y)
+            self.dispatch_event('on_mouse_enter', x / self._mouse_scale, y / self._mouse_scale)
             self._tracking = True
             track = TRACKMOUSEEVENT()
             track.cbSize = sizeof(track)
@@ -1015,10 +1071,10 @@ class Win32Window(BaseWindow):
             # Drag event
             modifiers = self._get_modifiers()
             self.dispatch_event('on_mouse_drag',
-                                x, y, dx, dy, buttons, modifiers)
+                                x / self._mouse_scale, y / self._mouse_scale, dx / self._mouse_scale, dy / self._mouse_scale, buttons, modifiers)
         else:
             # Motion event
-            self.dispatch_event('on_mouse_motion', x, y, dx, dy)
+            self.dispatch_event('on_mouse_motion', x / self._mouse_scale, y / self._mouse_scale, dx * self._mouse_scale, dy * self._mouse_scale)
         return 0
 
     @ViewEventHandler
@@ -1032,7 +1088,7 @@ class Win32Window(BaseWindow):
         self._tracking = False
         self._mouse_in_window = False
         self.set_mouse_platform_visible()
-        self.dispatch_event('on_mouse_leave', x, y)
+        self.dispatch_event('on_mouse_leave', x / self._mouse_scale, y / self._mouse_scale)
         return 0
 
     def _event_mousebutton(self, ev: str, button: int, lParam: int) -> int:
@@ -1042,7 +1098,7 @@ class Win32Window(BaseWindow):
             _user32.ReleaseCapture()
         x, y = self._get_location(lParam)
         y = self._height - y
-        self.dispatch_event(ev, x, y, button, self._get_modifiers())
+        self.dispatch_event(ev, x / self._mouse_scale, y / self._mouse_scale, button, self._get_modifiers())
         return 0
 
     @ViewEventHandler
@@ -1155,7 +1211,7 @@ class Win32Window(BaseWindow):
             self._update_clipped_cursor()
 
         self.switch_to()
-        self.dispatch_event('on_resize', self._width, self._height)
+        self.dispatch_event('_on_internal_resize', self._width, self._height)
         return 0
 
     @Win32EventHandler(constants.WM_SYSCOMMAND)
@@ -1170,7 +1226,7 @@ class Win32Window(BaseWindow):
     @Win32EventHandler(constants.WM_MOVE)
     def _event_move(self, msg: int, wParam: int, lParam: int) -> int:
         x, y = self._get_location(lParam)
-        self.dispatch_event('on_move', x, y)
+        self.dispatch_event('on_move', x / self._mouse_scale, y / self._mouse_scale)
         return 0
 
     @Win32EventHandler(constants.WM_SETCURSOR)
@@ -1245,11 +1301,10 @@ class Win32Window(BaseWindow):
 
         if self._minimum_size:
             info.ptMinTrackSize.x, info.ptMinTrackSize.y = \
-                self._client_to_window_size(*self._minimum_size)
+                self._client_to_window_size_dpi(*self._minimum_size)
         if self._maximum_size:
             info.ptMaxTrackSize.x, info.ptMaxTrackSize.y = \
-                self._client_to_window_size(*self._maximum_size)
-
+                self._client_to_window_size_dpi(*self._maximum_size)
         return 0
 
     @Win32EventHandler(constants.WM_ERASEBKGND)
@@ -1293,5 +1348,60 @@ class Win32Window(BaseWindow):
         self.dispatch_event('on_file_drop', point.x, self._height - point.y, paths)
         return 0
 
+    @Win32EventHandler(constants.WM_GETDPISCALEDSIZE)
+    def _event_dpi_scaled_size(self, msg: int, wParam: int, lParam: int) -> int | None:
+        if pyglet.options.dpi_scaling in ("scaled", "stretch"):
+            return None
+
+        size = cast(lParam, POINTER(SIZE)).contents
+
+        dpi = wParam
+
+        if constants.WINDOWS_10_CREATORS_UPDATE_OR_GREATER:
+            current = RECT()
+            result = RECT()
+
+            # Size between current size and future.
+            _user32.AdjustWindowRectExForDpi(byref(current),
+                                             self._ws_style, False, self._ex_ws_style,
+                                             _user32.GetDpiForWindow(self._hwnd))
+
+            _user32.AdjustWindowRectExForDpi(byref(result),
+                                             self._ws_style, False, self._ex_ws_style, dpi)
+
+            size.cx += (result.right - result.left) - (current.right - current.left)
+            size.cy += (result.bottom - result.top) - (current.bottom - current.top)
+            return 1
+
+        return None
+
+    @Win32EventHandler(constants.WM_DPICHANGED)
+    def _event_dpi_change(self, msg: int, wParam: int, lParam: int) -> int:
+        y_dpi, x_dpi = self._get_location(wParam)
+
+        scale = x_dpi / constants.USER_DEFAULT_SCREEN_DPI
+
+        self._dpi = x_dpi
+
+        if not self._fullscreen and \
+                (pyglet.options.dpi_scaling != "real" or constants.WINDOWS_10_CREATORS_UPDATE_OR_GREATER):
+            suggested_rect = cast(lParam, POINTER(RECT)).contents
+
+            x = suggested_rect.left
+            y = suggested_rect.top
+            width = suggested_rect.right - suggested_rect.left
+            height = suggested_rect.bottom - suggested_rect.top
+
+            _user32.SetWindowPos(self._hwnd, 0, x, y, width, height,
+                                 constants.SWP_NOZORDER | constants.SWP_NOOWNERZORDER | constants.SWP_NOACTIVATE)
+
+        if pyglet.options.dpi_scaling == "stretch":
+            self._mouse_scale = scale
+
+        self.switch_to()
+        self.dispatch_event('_on_internal_scale', scale, x_dpi)
+        return 1
+
+
 
 __all__ = ['Win32EventHandler', 'Win32Window']
